package com.jiuqi.util.patterns;

import java.util.ArrayList;
import java.util.List;

public final class Wildcard {
   private String pattern;
   private List items;
   public static final char DEFAULT_WILDCHAR = '%';

   public Wildcard(String pattern) {
      this(pattern, '%');
   }

   public Wildcard(String pattern, char wildchar) {
      this.items = new ArrayList();
      this.pattern = pattern;
      if (pattern != null && pattern.length() != 0) {
         int cur = 0;
         StringBuffer buffer = new StringBuffer(pattern.length());

         while(true) {
            while(true) {
               while(cur < pattern.length()) {
                  char ch = pattern.charAt(cur);
                  if (ch == wildchar) {
                     if (cur < pattern.length() - 1 && pattern.charAt(cur + 1) == wildchar) {
                        buffer.append(wildchar);
                        cur += 2;
                     } else {
                        if (buffer.length() > 0) {
                           this.items.add(buffer.toString().toUpperCase());
                           buffer.setLength(0);
                        }

                        this.items.add((Object)null);
                        ++cur;
                     }
                  } else {
                     buffer.append(ch);
                     ++cur;
                  }
               }

               if (buffer.length() > 0) {
                  this.items.add(buffer.toString().toUpperCase());
               }

               return;
            }
         }
      }
   }

   public boolean match(String s) {
      return s != null && this.items.size() != 0 ? this.matchFrom(s.toUpperCase(), 0, 0) : false;
   }

   private boolean matchFrom(String s, int level, int fromIndex) {
      while(level < this.items.size()) {
         String subStr = (String)this.items.get(level);
         if (subStr != null) {
            do {
               if (fromIndex >= s.length()) {
                  return false;
               }

               int next = s.indexOf(subStr, fromIndex);
               if (next == -1 || level == 0 && next > 0) {
                  return false;
               }

               fromIndex = next + subStr.length();
            } while(!this.matchFrom(s, level + 1, fromIndex));

            return true;
         }

         ++level;
      }

      if (fromIndex < s.length()) {
         if (this.items.get(this.items.size() - 1) == null) {
            return true;
         } else {
            return false;
         }
      } else {
         return true;
      }
   }

   public String toString() {
      return this.pattern;
   }

   public static boolean match(String s, String pattern) {
      Wildcard wildcard = new Wildcard(pattern);
      return wildcard.match(s);
   }

   public static boolean match(String s, String pattern, char wildchar) {
      Wildcard wildcard = new Wildcard(pattern, wildchar);
      return wildcard.match(s);
   }
}
